﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Diagnostics;

namespace ActiveConsole
{
    class Program
    {
        private static PerformanceCounter p = new PerformanceCounter("Processor", "% Processor Time", "_Total");
        /// <summary>
        /// 防止自动刷新和手动刷新同时执行
        /// </summary>
        private static Mutex mutex = new Mutex();
        /// <summary>
        /// 用List构建队列
        /// </summary>
        private static List<int> loadingHistory = new List<int>();

        static void Main(string[] args)
        {
            // 定义一些窗口信息
            Console.CursorVisible = false;
            Console.WindowHeight = 18;
            Console.WindowWidth = 80;

            // 定时刷新一下
            Timer timer = new Timer(new TimerCallback(Refresh), null, 10, 2 * 1000);

            // 先画出一个框架
            BuildFramework();

            // 按一下键盘就刷新一下
            while (Console.ReadKey(true).Key != ConsoleKey.Q)
            {
                Refresh(null);
            }
            timer.Dispose();
            p.Dispose();
        }

        private static void BuildFramework()
        {
            Console.WriteLine(@"                                  CPU LOADING                                ");// 0
            Console.WriteLine(@"                                  ~~~~~~~~~~~                                ");// 1
            Console.WriteLine(@" 11% [                                                                      ]");// 2
            Console.WriteLine(@"     ========================================================================");// 3  
            Console.WriteLine(@"100% +----------------------------------------------------------------------+");// 4   +
            Console.WriteLine(@" 90% +----------------------------------------------------------------------+");// 5   |
            Console.WriteLine(@" 80% +----------------------------------------------------------------------+");// 6   |
            Console.WriteLine(@" 70% +----------------------------------------------------------------------+");// 7   |
            Console.WriteLine(@" 60% +----------------------------------------------------------------------+");// 8   |   
            Console.WriteLine(@" 50% +----------------------------------------------------------------------+");// 9   yLenth:11
            Console.WriteLine(@" 40% +----------------------------------------------------------------------+");// 10  |
            Console.WriteLine(@" 30% +----------------------------------------------------------------------+");// 11  |
            Console.WriteLine(@" 20% +----------------------------------------------------------------------+");// 12  |
            Console.WriteLine(@" 10% +----------------------------------------------------------------------+");// 13  |
            Console.WriteLine(@"  0% +----------------------------------------------------------------------+");// 14  +
            Console.WriteLine(@"     ========================================================================");// 15
            //                  |    |    |    |    |    |    |    |    |    |    |    |    |    |    |    | 
            //                  0    5   10   15   20   25   30   35   40   45   50   55   60   65   70   75  
            //                        |<----------------------- xLength:70 -------------------------------->|           
        }
        private static void Refresh(object o)
        {
            mutex.WaitOne();

            int xLength = 70;
            int yLength = 11;
            
            // 11% 替换为CPU的负载 
            string loading = string.Format("{0:0}", p.NextValue());
            loading = string.Format("{0,3}", loading);
            Console.SetCursorPosition(0, 2);
            Console.Write(loading);

            // 用#画出类似ProcessBar的效果
            Console.SetCursorPosition(6, 2);
            int graphics = int.Parse(loading) * xLength / 100;
            for (int i = 0; i < graphics; i++)
            {
                Console.Write(@"#");
            }
            for (int i = 0; i < xLength - graphics; i++)
            {
                Console.Write(@" ");
            }

            
            // 用List维护一个最大长度为xLength的队列
            loadingHistory.Add(graphics);
            if (loadingHistory.Count > xLength)
            {
                loadingHistory.RemoveAt(0);
            }

            // 画出CPU占用的柱状图
            for (int i = 0; i < loadingHistory.Count; i++)
            {
                int waterline = loadingHistory[i] / yLength;
                if (waterline >= 0)
                {
                    for (int j = 0; j < yLength; j++)
                    {
                        Console.SetCursorPosition(6 + i, 14 - j);
                        if (j - waterline <= waterline)
                        {
                            Console.Write("#");
                        }
                        else
                        {
                            Console.Write("-");
                        }
                    }
                }
            }

            mutex.ReleaseMutex();
        }
    }
}
